/************************************************************************/
/*  cvthold -	convert short holdings record between ascii and binary.	*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2002 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static char sccsid[] = "$Id: cvthold.c,v 1.5 2007/08/29 18:28:29 doug Exp $ ";
#endif

#include <stdio.h>
#include "version.h"
   
char *syntax[] = {
"%s version " VERSION,
"%s -i a|b -o a|b [input_file [output_file]]",
"    where:",
"	-i a|b	    Input format is either ascii or binary format.",
"	-o a|b	    Output format is either ascii or binary format.",
"	-N net	    Specify network field.",
"	input_file  specifies optional input file.  If no file is specified,",
"		    input is read from stdin.",
"	output_file specifies optional output file.  If no file is specified,",
"		    output is written to stdout.",
NULL };

/************************************************************************/

#include <stdlib.h>
#include <strings.h>

#define	EXTERN

#include "qlib2.h"

#include "holdings.h"

char *input, *output;
char *infile, *outfile;
char *infmt, *outfmt;
char *network;
char *cmdname;
FILE *fpin, *fpout, *info;
int date_fmt;

/************************************************************************/
/*  read_hold:								*/
/*	read the next HOLDING_SHORT record from the specified FILE.	*/
/*  return:								*/
/*	ptr to static HOLDING_SHORT record.				*/
/************************************************************************/
HOLDING_SHORT *read_hold 
   (FILE	*fp,		/* input FILE.				*/
    char	*fmt)		/* format specifier: "a" or "b".	*/
{
    INT_TIME it, *p_it;
    static HOLDING_SHORT hr;
    int len, version, blkexp, data_type,  sample_rate, sample_rate_mult;
    int offset, length;
    int n;
    char line[160], type_str[4], start_str[40], end_str[40];
    switch (*fmt) {
      case 'a':
	if (fgets(line,160,fp)==NULL) return (NULL);
	if ((n=sscanf(line, "%d %d %d %d %s %s %s %s %d %d %s %s %d %d ", 
		      &len, &version, &blkexp, &data_type,
		      hr.station, hr.network, hr.channel, hr.location, 
		      &sample_rate, &sample_rate_mult,
		      start_str, end_str, 
		      &offset, &length)) != 14) {
	    fprintf (stderr, "Error - %d items found on input line\n", n);
	    exit(1);
	}
	hr.len = len;
	hr.version = version;
	hr.blkexp = blkexp;
	hr.data_type = data_type;
	hr.sample_rate = sample_rate;
	hr.sample_rate_mult = (sample_rate_mult) ? sample_rate_mult : 1;
	if ((p_it=parse_date(start_str))) {
	    hr.start_time = encode_time_sdr(*p_it,my_wordorder);
	}
	else {
	    fprintf (stderr, "Invalid start time: %s\n", start_str);
	    exit(1);
	}
	if ((p_it=parse_date(end_str))) {
	    hr.end_time = encode_time_sdr(*p_it,my_wordorder);
	}
	else {
	    fprintf (stderr, "Invalid end time: %s\n", end_str);
	    exit(1);
	}
	hr.offset = offset;
	hr.length = length;
	/* Remove "_" or "-" in network and location field. */
	if (hr.network[0]=='_' || hr.network[0]=='-') hr.network[0] = '\0';
	if (hr.location[0]=='_' || hr.location[0]=='-') hr.location[0] = '\0';
	return (&hr);
	break;
      case 'b':	
	n = fread (&hr, sizeof(HOLDING_SHORT), 1, fp);
	if (n && hr.sample_rate_mult == 0) hr.sample_rate_mult = 1;
	return ((n) ? &hr : NULL);
	break;
      default:	
	return (NULL); 
	break;
    }
}

/************************************************************************/
/*  write_hold:								*/
/*	write the HOLDING_SHORT record to the specified FILE.		*/
/*  return:								*/
/*	0 on success, -1 on error.					*/
/************************************************************************/
int write_hold
   (FILE	*fp,		/* output FILE.				*/
    char	*fmt,		/* format specifier: "a" or "b".	*/
    HOLDING_SHORT *ph)		/* ptr to record to write.		*/
{
    int n;
    if (network) strncpy(ph->network,network,2);
    switch (*fmt) {
      case 'a':
	/* Substitute "-" for empty network and location field. */
	if (ph->network[0]=='\0') ph->network[0] = '-';
	if (ph->location[0]=='\0') ph->location[0] = '-';
	printf ("%d %d %d %d %-5.5s %-2.2s %-3.3s %-2.2s %5d %5d %s ", 
		ph->len, ph->version, ph->blkexp, ph->data_type,
		ph->station, ph->network, ph->channel, ph->location, 
		ph->sample_rate, ph->sample_rate_mult,
		time_to_str(decode_time_sdr(ph->start_time,my_wordorder), date_fmt));
	printf (" %s", time_to_str(decode_time_sdr(ph->end_time,my_wordorder), date_fmt));
	printf (" %d %d\n", ph->offset, ph->length);
	return (0);
	break;
      case 'b':	
	fwrite (ph, sizeof(HOLDING_SHORT), 1, fp);
	return (0);
	break;
      default:	return (-1);
    }
}

/************************************************************************/
/*  main:   main program.						*/
/************************************************************************/
int main (int argc, char **argv)
{
    /* Variables needed for getopt. */
    extern char	*optarg;
    extern int	optind, opterr;
    int		c;

    HOLDING_SHORT *hr;
    char *date_fmt_str = NULL;
    int n;
    char *p;

    /*	Initialization.							*/
    infile = "stdin";
    fpin = stdin;		/* assume input to come from stdin.	*/
    outfile = "stdout";		/* empty initial output directory.	*/
    info = stdout;
    date_fmt = JULIAN_FMT;
    get_my_wordorder ();
    cmdname = trim(*argv);

    /*	Parse command line options.					*/
    while ( (c = getopt(argc,argv,"hi:o:N:D:")) != -1)
	switch (c) {
	case '?':
	case 'h':   print_syntax(cmdname,syntax,info); exit(0); break;
	case 'i':   infmt=optarg; break;
	case 'o':   outfmt=optarg; break;
	case 'N':   network = optarg; break;
	case 'D':   date_fmt_str = optarg; break;
	}

    if (infmt == NULL || outfmt == NULL) {
	fprintf (stderr, "Missing -i or -o option or argument\n");
	exit(1);
    }
    lowercase(infmt);
    lowercase(outfmt);
    if (network) uppercase(network);
    switch (*infmt) {
      case 'a':
      case 'b':	break;
      default:	fprintf (stderr, "Invalid -i argument: %s\n", infmt);
		exit(1);
    }
    switch (*outfmt) {
      case 'a':
      case 'b':	break;
      default:	fprintf (stderr, "Invalid -o argument: %s\n", outfmt);
		exit(1);
    }
    if (date_fmt_str) date_fmt = date_fmt_num(date_fmt_str);
    if (date_fmt%2 == 0) ++date_fmt;

    /*	Skip over all options and their arguments.			*/
    argv += optind;
    argc -= optind;

    /* The remaining arguments are the input files.			*/
    fpin = stdin;
    if (argc-- > 0) {
	info = stderr;
	infile = *argv++;
	if ((fpin = fopen(infile,"r")) == NULL) {
	    fprintf (stderr, "error opening input file %s\n", infile);
	    exit(1);
	}
    }
    fpout = stdout;
    if (argc-- > 0) {
	info = stdout;
	outfile = *argv++;
	if ((fpout = fopen(outfile,"w")) == NULL) {
	    fprintf (stderr, "error opening output file %s\n", outfile);
	    exit(1);
	}
	info = stdout;
    }

    
    while (hr = read_hold(fpin,infmt)) {
	n = write_hold (fpout, outfmt, hr);
	if (n != 0) {
	    fprintf (stderr, "error writing output\n");
	    exit(1);
	}
    }
}

