/************************************************************************/
/*  cvthold -	convert short holdings record between ascii and binary.	*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2000 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static char sccsid[] = "$Id: cvthold3to4.c,v 1.4 2007/08/29 18:28:29 doug Exp $ ";
#endif

#include <stdio.h>
#include "version.h"
   
char *syntax[] = {
"%s version " VERSION,
"%s -i a|b -o a|b [-M mapping_file] [-N net] [-D date_fmt] [input_file [output_file]]",
"    where:",
"	-i a|b	    Input format is either ascii or binary format.",
"	-o a|b	    Output format is either ascii or binary format.",
"	-M mapping_file",
"		    Specifies a mapping file that provides datatype",
"		    information for specific channels.",
"	-N net	    Specify network field.",
"	-D date_fmt Specify date format.",
"	input_file  specifies optional input file.  If no file is specified,",
"		    input is read from stdin.",
"	output_file specifies optional output file.  If no file is specified,",
"		    output is written to stdout.",
NULL };

/************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <errno.h>

#define	EXTERN

#include "qlib2.h"

#include "holdings.h"

char *input, *output;
char *infile, *outfile;
char *infmt, *outfmt;
char *network;
char *cmdname;
char *mapping_file;
FILE *fpin, *fpout, *info;
int date_fmt;
typedef struct _map {
    char station[6];
    char network[3];
    char channel[4];
    char location[3];
    int data_type;
} MAP;
int nmap;
MAP *map;

/************************************************************************/
/*  93/06/20	Doug Neuhauser, Seismographic Station, UC Berkeley	*/
/*									*/
/*  After discussion with Tim Ahern at IRIS, we have changed the	*/
/*  location field to the network field in the holding and request	*/
/*  file.  This will hold the 2 character NETWORK_CODE associated with	*/
/*  SEED V2.3 Fixed Data Headers.					*/
/************************************************************************/

typedef struct _holding_short3 {
    char    station[HR_STATION_LEN+1];		/* name of station.			*/
    char    network[HR_NETWORK_LEN+1];		/* station location.			*/
    char    channel[HR_CHANNEL_LEN+1];		/* SEED channel name.			*/
    char    type;				/* 'S' = sdr time.			*/
    short int sample_rate;			/* sample rate (+=samp/sec, -=sec/samp)	*/
    SDR_TIME start_time;			/* start time: yyyy,ddd,hh:mm:ss.tttt	*/
    short int sample_rate_mult;			/* padding.				*/
    SDR_TIME end_time;				/* end time: yyyy,ddd,hh:mm:ss.tttt	*/
    short   pad2;				/* padding.				*/
} HOLDING_SHORT3;

/************************************************************************/
/*  hrcpy3:								*/
/*	Copy HOLDING_SHORT3 record to HOLDING_SHORT record.		*/
/************************************************************************/
HOLDING_SHORT *hrcpy3 ( HOLDING_SHORT *hr, HOLDING_SHORT3 *hr3)
{
    int i;
    memset (hr, 0, sizeof(HOLDING_SHORT));
    hr->len = sizeof (HOLDING_SHORT);
    hr->version = HOLDING_SHORT_VERSION;
    hr->blkexp = 12;			/* UNKNOWN - ASSUMPTION */
    hr->data_type = -1;			/* UNKNOWN */
    strcpy (hr->station, hr3->station);
    strcpy (hr->network, hr3->network);
    strcpy (hr->channel, hr3->channel);
    /* Location field is empty. */	/* ASSUMPTION */
    hr->sample_rate = hr3->sample_rate;
    hr->sample_rate_mult = hr3->sample_rate_mult;
    hr->start_time = hr3->start_time;
    hr->end_time = hr3->end_time;
    hr->offset = -1;			/* UNKNOWN */
    hr->length = -1;			/* UNKNOWN */
    if (nmap) {
	for (i=0; i<nmap; i++) {
	    if (fnmatch(map[i].station,hr->station)==0 &&
		fnmatch(map[i].network,hr->network)==0 &&
		fnmatch(map[i].channel,hr->channel)==0 &&
		fnmatch(map[i].location,hr->location)==0) {
		hr->data_type = map[i].data_type;
		break;
	    }
	}
    }
    return (hr);
}

/************************************************************************/
/*  read_hold3:								*/
/*	read the next HOLDING_SHORT3 record from the specified FILE.	*/
/*  return:								*/
/*	ptr to static HOLDING_SHORT record.				*/
/************************************************************************/
HOLDING_SHORT *read_hold3 
   (FILE	*fp,		/* input FILE.				*/
    char	*fmt)		/* format specifier: "a" or "b".	*/
{
    INT_TIME it, *p_it;
    static HOLDING_SHORT hr;
    static HOLDING_SHORT3 hr3;
    int n, rate, rate_mult;
    char line[160], type_str[4], start_str[40], end_str[40];
    switch (*fmt) {
      case 'a':
	if (fgets(line,160,fp)==NULL) return (NULL);
	if ((n=sscanf(line,"%s %s %s %d %d %s %s", hr3.station, hr3.network, hr3.channel,
		  &rate, &rate_mult, start_str, end_str)) != 7) {
	    fprintf (stderr, "Error - %d items found on input line\n", n);
	    exit(1);
	}
	hr3.type = 'S';
	hr3.sample_rate = rate;
	hr3.sample_rate_mult = (rate_mult) ? rate_mult : 1;
	if ((p_it=parse_date(start_str))) {
	    hr3.start_time = encode_time_sdr(*p_it,my_wordorder);
	}
	else {
	    fprintf (stderr, "Invalid start time: %s\n", start_str);
	    exit(1);
	}
	if ((p_it=parse_date(end_str))) {
	    hr3.end_time = encode_time_sdr(*p_it,my_wordorder);
	}
	else {
	    fprintf (stderr, "Invalid end time: %s\n", end_str);
	    exit(1);
	}
	hrcpy3(&hr,&hr3);
	return (&hr);
	break;
      case 'b':	
	n = fread (&hr3, sizeof(HOLDING_SHORT3), 1, fp);
	if (n && hr3.sample_rate_mult == 0) hr3.sample_rate_mult = 1;
	hrcpy3(&hr,&hr3);
	return ((n) ? &hr : NULL);
	break;
      default:	
	return (NULL); 
	break;
    }
}

/************************************************************************/
/*  write_hold:								*/
/*	write the HOLDING_SHORT record to the specified FILE.		*/
/*  return:								*/
/*	0 on success, -1 on error.					*/
/************************************************************************/
int write_hold
   (FILE	*fp,		/* output FILE.				*/
    char	*fmt,		/* format specifier: "a" or "b".	*/
    HOLDING_SHORT *ph)		/* ptr to record to write.		*/
{
    int n;
    if (network) strncpy(ph->network,network,2);
    switch (*fmt) {
      case 'a':
	/* Substitute "-" for empty network and location field. */
	if (ph->network[0]=='\0') ph->network[0] = '-';
	if (ph->location[0]=='\0') ph->location[0] = '-';
	fprintf (fp, "%d %d %d %d %-5.5s %-2.2s %-3.3s %-2.2s %4d %4d %s ", 
		 ph->len, ph->version, ph->blkexp, ph->data_type,
		 ph->station, ph->network, ph->channel, ph->location, 
		 ph->sample_rate, ph->sample_rate_mult,
		 time_to_str(decode_time_sdr(ph->start_time,my_wordorder), date_fmt));
	fprintf (fp, " %s", time_to_str(decode_time_sdr(ph->end_time,my_wordorder), date_fmt));
	fprintf (fp, " %d %d\n", ph->offset, ph->length);
	return (0);
	break;
      case 'b':	
	fwrite (ph, sizeof(HOLDING_SHORT), 1, fp);
	return (0);
	break;
      default:	return (-1);
    }
}

/************************************************************************/
/*  read_map:								*/
/*	Read mapping file.						*/
/************************************************************************/
int read_map (char *file)
{
    int n;
    FILE *fp;
    char line[80];
    if ((fp=fopen(file,"r"))==NULL) {
	fprintf (stderr, "Error %d opening mapping file %s\n", errno, file);
	exit(1);
    }
    while (fgets(line,80,fp)) {
	if (line[0] == '#') continue;
	map = (map==NULL) ? (MAP *)malloc((nmap+1)*sizeof(MAP)) :
	    (MAP *)realloc(map,(nmap+1)*sizeof(MAP));
	if (map==NULL) {
	    fprintf (stderr, "Error mallocing mapping info\n");
	    exit(1);
	}
	if ((n=sscanf(line, "%s %s %s %s %d", 
		     map[nmap].station, map[nmap].network,
		     map[nmap].channel, map[nmap].location,
		     &map[nmap].data_type)) != 5) {
	    fprintf (stderr, "Error reading map line: %s", line);
	    exit(1);
	}
	nmap++;
    }
    fclose (fp);
    return (nmap);
}

/************************************************************************/
/*  main:   main program.						*/
/************************************************************************/
int main (int argc, char **argv)
{
    /* Variables needed for getopt. */
    extern char	*optarg;
    extern int	optind, opterr;
    int		c;

    HOLDING_SHORT *hr;
    char *date_fmt_str = NULL;
    int n;
    char *p;

    /*	Initialization.							*/
    infile = "stdin";
    fpin = stdin;		/* assume input to come from stdin.	*/
    outfile = "stdout";		/* empty initial output directory.	*/
    info = stdout;
    date_fmt = JULIAN_FMT;
    get_my_wordorder ();
    cmdname = trim(*argv);

    /*	Parse command line options.					*/
    while ( (c = getopt(argc,argv,"hi:o:N:D:M:")) != -1)
	switch (c) {
	case '?':
	case 'h':   print_syntax(cmdname,syntax,info); exit(0); break;
	case 'i':   infmt=optarg; break;
	case 'o':   outfmt=optarg; break;
	case 'N':   network = optarg; break;
	case 'D':   date_fmt_str = optarg; break;
	case 'M':   mapping_file = optarg; break;
	}

    if (infmt == NULL || outfmt == NULL) {
	fprintf (stderr, "Missing -i or -o option or argument\n");
	exit(1);
    }
    lowercase(infmt);
    lowercase(outfmt);
    if (network) uppercase(network);
    switch (*infmt) {
      case 'a':
      case 'b':	break;
      default:	fprintf (stderr, "Invalid -i argument: %s\n", infmt);
		exit(1);
    }
    switch (*outfmt) {
      case 'a':
      case 'b':	break;
      default:	fprintf (stderr, "Invalid -o argument: %s\n", outfmt);
		exit(1);
    }
    if (date_fmt_str) date_fmt = date_fmt_num(date_fmt_str);
    if (date_fmt%2 == 0) ++date_fmt;

    if (mapping_file) read_map(mapping_file);

    /*	Skip over all options and their arguments.			*/
    argv += optind;
    argc -= optind;

    /* The remaining arguments are the input files.			*/
    fpin = stdin;
    if (argc-- > 0) {
	info = stderr;
	infile = *argv++;
	if ((fpin = fopen(infile,"r")) == NULL) {
	    fprintf (stderr, "error opening input file %s\n", infile);
	    exit(1);
	}
    }
    fpout = stdout;
    if (argc-- > 0) {
	info = stdout;
	outfile = *argv++;
	if ((fpout = fopen(outfile,"w")) == NULL) {
	    fprintf (stderr, "error opening output file %s\n", outfile);
	    exit(1);
	}
	info = stdout;
    }

    
    while (hr = read_hold3(fpin,infmt)) {
	n = write_hold (fpout, outfmt, hr);
	if (n != 0) {
	    fprintf (stderr, "error writing output\n");
	    exit(1);
	}
    }
}

