/************************************************************************/
/*  Routines to process current input record.				*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2000 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static char sccsid[] = "$Id: process_block.c,v 1.7 2007/08/30 03:19:26 doug Exp $ ";
#endif

#include    <stdio.h>
#include    <stdlib.h>
#include    <strings.h>
#include    <math.h>

#include    "qlib2.h"

#include    "params.h"
#include    "st_info.h"
#include    "holdings.h"
#include    "procs.h"
#include    "externals.h"

#define	    SIGN(i)	( (i>=0) ? 1 : -1 )
#define	    OUTPUT_BLKSIZE	    4096

/************************************************************************/
/*  process_block:							*/
/*	Process input block for output.					*/
/************************************************************************/
void process_block
   (ST_INFO	*st_p)		/* ptr to output stream structure.	*/
{
    INT_TIME	    exptime;
    ST_INFO	    *out = st_head_out.next;
    int		    seconds, usecs;
    int		    usecs_per_point;
    double	    bdif;

reprocess:
    /*  Ensure that all headers for the output stream are allocated     */
    /*  and initialized.                                                */
    if (out->sum_hdr == NULL) make_out_sum_hdr(out, st_p);
    
    if (out->cur_hdr == NULL) make_out_cur_hdr(out);

    /*  Perform sanity checks on the input stream.                      */
    if (! same_type_data_stream (st_p->cur_hdr, out->sum_hdr)) {
	if (multi_channel_flag) {
	    /*  Discontiguous data stream.                              */
	    /*  Flush current output and start a new section.   */
	    /* Update initial offset to point to the current record.	*/
	    out->init_offset = st_p->init_offset;
	    out->cur_offset = st_p->cur_offset;
	    flush_output(out);
	    print_stream_summary(out);
	    free_data_hdr (out->cur_hdr);
	    out->cur_hdr = NULL;
	    free_data_hdr (out->sum_hdr);
	    out->sum_hdr = NULL;
	    out->station[0] = out->channel[0] = out->network[0] = out->location[0] = '\0';
	    out->rate = 0;
	    out->rate_mult = 0;
	    out->num_blk = 0;
	    out->total_slew = out->max_slew = out->min_slew = out->max_step = 0; 
	    goto reprocess;
	}
	else {
	    fprintf (info, "Error: input block for different stream than output stream\n");
	    fprintf (info, "output: %s.%s.%s.%s,(%d,%d)  input:  %s.%s.%s.%s,(%d,%d)\n",
		     out->sum_hdr->station_id, out->sum_hdr->network_id,
		     out->sum_hdr->channel_id, out->sum_hdr->location_id,
		     out->sum_hdr->sample_rate, out->sum_hdr->sample_rate_mult,
		     st_p->cur_hdr->station_id, st_p->cur_hdr->network_id,
		     st_p->cur_hdr->channel_id, st_p->cur_hdr->location_id,
		     st_p->cur_hdr->sample_rate, st_p->cur_hdr->sample_rate_mult);
            exit(1);
	}
    }

    if (contiguous_flag) {
	/*  Generate a separate holdings entry for each contiguous 	*/
	/*  datastream within a file.					*/
	if (out->cur_hdr->num_samples > 0) {
	    time_interval2 (1,st_p->cur_hdr->sample_rate,st_p->cur_hdr->sample_rate_mult,
			    &seconds,&usecs);
	    usecs_per_point = seconds * USECS_PER_SEC + usecs;
	    exptime = add_time (out->cur_hdr->endtime, seconds, usecs);
	    bdif = tdiff (st_p->cur_hdr->begtime, exptime);
	    if (bdif >= TOL(st_p->cur_hdr->sample_rate,st_p->cur_hdr->sample_rate_mult)) {
		/*  Discontiguous data stream.				*/
		/*  Flush current output and start a new section.	*/
		out->init_offset = st_p->init_offset;
		out->cur_offset = st_p->cur_offset;
		flush_output(out);
		print_stream_summary(out);
		free_data_hdr (out->cur_hdr);
		out->cur_hdr = NULL;
		free_data_hdr (out->sum_hdr);
		out->sum_hdr = NULL;
		goto reprocess;
	    }
	}
	else {
	    st_p->init_offset = st_p->cur_offset;
	}
    }
	

    append_to_output (st_p, bdif);

    /*	Update input header structures.					*/
    if (st_p->prev_hdr != NULL) free_data_hdr (st_p->prev_hdr);
    /* Increment file offset past this record. */
    st_p->cur_offset += st_p->cur_hdr->blksize;
    st_p->prev_hdr = st_p->cur_hdr;
    st_p->cur_hdr = NULL;
    return;
}

/************************************************************************/
/*  flush_output:							*/
/*	Flush any buffered output.					*/
/************************************************************************/
void flush_output
   (ST_INFO	*out)		/* ptr to output stream structure.	*/
{
    IOB		    *iob = out->iob;
    DATA_HDR	    *ch = out->cur_hdr;

    if (ch != NULL && ch->num_samples > 0) write_holdings (out);
}

/************************************************************************/
/*  append_to_output:							*/
/*	Append input to output.						*/
/************************************************************************/
void append_to_output 
   (ST_INFO	*st_p,		/* ptr to output stream structure.	*/
    int		slew)		/* slew between records.		*/
{
    ST_INFO	*out = st_head_out.next;
    DATA_HDR	*oh = out->cur_hdr;
    DATA_HDR	*ih = st_p->cur_hdr;
    DATA_HDR	*sh = out->sum_hdr;

    out->blksize = ih->blksize;
    out->data_type = ih->data_type;

    /*	Update the flags and other header info (including flags).	*/
    oh->activity_flags |= ih->activity_flags;
    oh->io_flags |= ih->io_flags;
    oh->data_quality_flags |= ih->data_quality_flags;
    if (oh->num_samples == 0) {
	oh->begtime = ih->begtime;
	oh->x0 = ih->x0;
    }
    oh->endtime = ih->endtime;
    oh->num_samples += ih->num_samples;
    oh->xn = ih->xn;

    /*	Update the input buffer pointers to point past this the input	*/
    /*	data block.							*/
    st_p->iob->avail += st_p->blksize;

    /*	Update the information in the output's sum_hdr.			*/
    /*	This header provides a running summary of information for data	*/
    /*	that has been directed to the output stream.  This INCLUDES	*/
    /*	data that is placed in the output buffer but not yet written.	*/
    /*	It is therefore updated NOW, NOT when the buffer is written.	*/
    /*	This provides a uniform place for begtime, endtime, num_samples,*/
    /*	and other info that is needed to reflect the state of data	*/
    /*	that has been directed for output.				*/
    if (sh->num_samples == 0) {
	sh->begtime = ih->begtime;
	sh->x0 = ih->x0;
    }
    sh->endtime = ih->endtime;
    sh->num_samples += ih->num_samples;
    sh->xn = ih->xn;

    /*	Keep track of time slew information.			*/
    out->num_blk++;
    out->total_slew += slew;
    out->max_slew = MAX(out->total_slew, out->max_slew);
    out->min_slew = MIN(out->total_slew, out->min_slew);
    out->max_step = MAX(out->max_step, abs(slew));
}

/************************************************************************/
/*  same_type_data_stream:						*/
/*	Boolean function to compare stream characteristics.		*/
/************************************************************************/
int same_type_data_stream 
   (DATA_HDR	*ih,		/* ptr to intput DATA_HDR.		*/
    DATA_HDR	*oh)		/* ptr to output DATA_HDR.		*/
{
    if ((strcmp(ih->station_id, oh->station_id) != 0) ||
	(strcmp(ih->channel_id, oh->channel_id) != 0) ||
	(strcmp(ih->location_id, oh->location_id) != 0) ||
	(ih->sample_rate != oh->sample_rate) ||
	(ih->sample_rate_mult != oh->sample_rate_mult)) return (0);
    else return (1);
}

/************************************************************************/
/*  make_out_sum_hdr:							*/
/*	Alloc and init summary output header struct.			*/
/************************************************************************/
void make_out_sum_hdr
   (ST_INFO	*out,		/* ptr to output stream structure.	*/
    ST_INFO	*st_p)		/* ptr to input stream structure.	*/
{
    /*	Make and fill in summary header if we don't have one yet.	*/
    if (out->sum_hdr == NULL) {
	out->sum_hdr = new_data_hdr();
	/*  Copy info from current input stream to summary header.	*/
	strcpy (out->sum_hdr->station_id, st_p->cur_hdr->station_id);
	strcpy (out->sum_hdr->channel_id, st_p->cur_hdr->channel_id);
	strcpy (out->sum_hdr->network_id, 
		(st_p->cur_hdr->network_id[0] == '\0' || st_p->cur_hdr->network_id[0] == ' ') ?
		network : st_p->cur_hdr->network_id);
	strcpy (out->sum_hdr->location_id, st_p->cur_hdr->location_id);
	out->sum_hdr->sample_rate = st_p->cur_hdr->sample_rate;
	out->sum_hdr->sample_rate_mult = st_p->cur_hdr->sample_rate_mult;
	/*  Fill in stream header structure also from summary header.	*/
	strcpy (out->station, out->sum_hdr->station_id);
	strcpy (out->network, out->sum_hdr->network_id);
	strcpy (out->channel, out->sum_hdr->channel_id);
	strcpy (out->location, out->sum_hdr->location_id);
	out->rate = out->sum_hdr->sample_rate;
	out->rate_mult = out->sum_hdr->sample_rate_mult;
	out->hdr_type = SDR_HDR_TYPE;
	out->blksize = st_p->blksize;
    }
}

/************************************************************************/
/*  make_out_cur_hdr:							*/
/*	Alloc and init current output header struct.			*/
/************************************************************************/
void make_out_cur_hdr
   (ST_INFO	*out)		/* ptr to output stream structure.	*/
{
    /*	Make and fill in current header if we don't have one yet.	*/
    if (out->cur_hdr == NULL) {
	out->cur_hdr = new_data_hdr();
	strcpy (out->cur_hdr->station_id, out->sum_hdr->station_id);
	strcpy (out->cur_hdr->channel_id, out->sum_hdr->channel_id);
	strcpy (out->cur_hdr->network_id, 
		(out->sum_hdr->network_id[0] == '\0' || out->sum_hdr->network_id[0] == ' ') ?
		network : out->sum_hdr->network_id);
	strcpy (out->cur_hdr->location_id, out->sum_hdr->location_id);
	out->cur_hdr->sample_rate = out->sum_hdr->sample_rate;
	out->cur_hdr->sample_rate_mult = out->sum_hdr->sample_rate_mult;
	out->cur_hdr->first_data = 64;
	out->cur_hdr->seq_no = (out->prev_hdr == NULL) ? 1 : 
	    out->prev_hdr->seq_no + 1;
    }
}
