/************************************************************************/
/*  qhold -	create holdings record for input data file.		*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2000 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static char sccsid[] = "$Id: qhold.c,v 1.5 2007/08/29 18:28:29 doug Exp $ ";
#endif

#include <stdio.h>
#include <strings.h>

#include "qlib2.h"
#include "version.h"
   
/*  Changeable defaults.	*/

#ifndef	DEFAULT_DCC
#define	DEFAULT_DCC		"BK"
#endif

#ifndef	DEFAULT_OPERATOR
#define	DEFAULT_OPERATOR	"XX"
#endif

#ifndef	DEFAULT_HEADER_DIR
#define	DEFAULT_HEADER_DIR	"HDR000"
#endif

#ifndef	DEFAULT_NETWORK
#define	DEFAULT_NETWORK		"BK"
#endif

#ifndef	DEFAULT_MODE
#define	DEFAULT_MODE		"C"
#endif

char *syntax[] = {
"%s version " VERSION,
"%ss [-D dcc] [-H header_dir] [-N network ] [-d n] [-h] [-O operator] -M T|C ",
"	[-C] -[S] -[A] [input_file_1 ... input_file_n]",
"    where:",
"	-D dcc	    specifies the (2 character) name of the DCC.",
"		    Default is " DEFAULT_DCC ".",
"	-H hdrdir   specifies the partial name of the header directory.",
"		    Default is " DEFAULT_HEADER_DIR ".",
"	-O operator specifies the (2 character) initials of the operator.",
"		    Default is " DEFAULT_OPERATOR ".",
"	-N network  specifies the (2 character) network name for V2.3 holdings",
"		    if network field is not specified in the station blockette.",
"		    If you do not want any default network, specify an empty string.",
"		    Default is " DEFAULT_NETWORK ".",
"	-M T|C	    specifies the mode (Triggered or Continuous) of the data stream.",
"		    Default is " DEFAULT_MODE ".",
"	-C	    Generate entry for each contiguous section of data.",
"	-S	    Generate short (abbreviated) holdings records.",
"	-A	    Generate ascii version of short holdings records.",
"	-m	    Input file may contain multiple data channels.",
"	-d n	    Debug output (OR of any of the following values:)",
"			1 = time slews",
"			2 = time info",
"			4 = block info",
"			8 = block headers",
"			16 = flags",
"			32 = input stream",
"			64 = output stream",
"	-h	    Help - prints syntax message.",
"	input_file_1 ... input_file_n",
"		    specifies the input data files to be scanned in order to",
"		    create holdings records.",
NULL };

/************************************************************************/
#define	    EXTERN

#include "params.h"
#include "st_info.h"
#include "holdings.h"
#include "procs.h"
#include "externals.h"

int main (int argc, char **argv)
{
    /*	Initialization.							*/
    outfile = NULL;		/* empty initial output directory.	*/
    fpin = stdin;		/* assume input to come from stdin.	*/
    info = stdout;
    date_fmt = JULIAN_FMT;
    header_dir = DEFAULT_HEADER_DIR;
    operator = DEFAULT_OPERATOR;
    dcc = DEFAULT_DCC;
    mode = DEFAULT_MODE;
    debug_option = 0;
    get_my_wordorder();

    parse_cmdline (&argc, &argv);
    if (date_fmt%2==0) ++date_fmt;
    if (network == NULL) network = strdup(DEFAULT_NETWORK);
    trim(network);

    while (argc-- > 0) {
	make_holdings (*argv++);
    }
    return(0);
}

/************************************************************************/
/*  print_stream_summary:						*/
/*	Print stream summary info.					*/
/************************************************************************/
void print_stream_summary
   (ST_INFO	*out_p)		/* ptr to output stream structure.	*/
{
    int		seconds, usecs;
    INT_TIME	endtime;

    if (out_p->num_samples <= 0) return;

    /*	Output summary information for the output stream.		*/
    time_interval2 (1, out_p->sum_hdr->sample_rate, out_p->sum_hdr->sample_rate_mult,
		    &seconds, &usecs);
    endtime = add_time (out_p->sum_hdr->endtime, seconds, usecs);
    fprintf (info, "%s.%s.%s.%s ",
	     out_p->sum_hdr->station_id, out_p->sum_hdr->network_id, 
	     out_p->sum_hdr->channel_id, out_p->sum_hdr->location_id);
    if (out_p->sum_hdr->sample_rate_mult == 1)
	fprintf (info, "rate=%d ", out_p->sum_hdr->sample_rate);
    else
	fprintf (info, "rate=%.4lf ", sps_rate(out_p->sum_hdr->sample_rate,
					       out_p->sum_hdr->sample_rate_mult));
    fprintf (info, "(%s to ",
	     time_to_str(out_p->sum_hdr->begtime,date_fmt));
    fprintf (info, "%s) : %d points, %.1lf msec correction, ", 
	     time_to_str(endtime,date_fmt), out_p->sum_hdr->num_samples, 
	     ((double)out_p->total_slew)/USECS_PER_MSEC);
    fprintf (info, "(min,max,max_step = %.1lf,%.1lf,%.1lf msec)\n",
	     ((double)out_p->min_slew)/USECS_PER_MSEC, 
	     ((double)out_p->max_slew)/USECS_PER_MSEC,
	     ((double)out_p->max_step)/USECS_PER_MSEC);
}
