/************************************************************************/
/*  Routines to write HOLDING_REC and HOLDING_SHORT records.		*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Seismological Laboratory					*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 1996-2000 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static char sccsid[] = "$Id: write_holdings.c,v 1.4 2007/08/29 18:28:29 doug Exp $ ";
#endif

#include <stdio.h>
#include <math.h>
#include <strings.h>
#include <time.h>

#include "qlib2.h"

#include "params.h"
#include "st_info.h"
#include "externals.h"
#include "holdings.h"
#include "procs.h"

#define log2(x)     ((double)(log(x)/M_LN2))

/************************************************************************/
/*  write_holdings:							*/
/*	Write holding record.						*/
/*  return:								*/
/*	0 on success, non-zero on failure.				*/
/************************************************************************/
int write_holdings
   (ST_INFO	*out)		/* ptr to output stream structure.	*/
{
    int status;
    if (! short_flag) {
	status = write_holdings_long(out);
    }
    else {
	status = write_holdings_short(out);
    }
    return (status);
}

/************************************************************************/
/*  write_holdings_long:						*/
/*	Write long holding record (HOLDING_REC).				*/
/*  return:								*/
/*	0 on success, non-zero on failure.				*/
/************************************************************************/
int write_holdings_long
   (ST_INFO	*out)		/* ptr to output stream structure.	*/
{
    HOLDING_REC hr;		/* holdings record.			*/
    DATA_HDR	*sh = out->sum_hdr;
    char	str[80];
    SDR_TIME	sdr_time;
    int		size_kb;
    char	*filename;
    char	*p;
    time_t	unix_time;

    filename = tail(out->filename);

    capntstr(hr.station, sh->station_id, HR_STATION_LEN);
    capntstr(hr.network, sh->network_id, HR_NETWORK_LEN);
    capntstr(hr.channel, sh->channel_id, HR_CHANNEL_LEN);
    sprintf (str,"%10.4f",sps_rate(sh->sample_rate,sh->sample_rate_mult));
    capntstr(hr.samples_per_sec, str, HR_SPS_LEN);
    capntstr(hr.mode, mode, HR_MODE_LEN);
    capntint(hr.num_segments, 0, HR_NUM_SEGMENTS_LEN);
    
    sdr_time = encode_time_sdr (sh->begtime,my_wordorder);
    asc_sdr_time (str,sdr_time,my_wordorder);
    capntstr(hr.start_time, str, HR_START_TIME_LEN);
    unix_time = unix_time_from_sdr_time (sdr_time,my_wordorder);
    capntlong(hr.unix_start_time, unix_time, HR_UNIX_START_LEN);
    
    sdr_time = encode_time_sdr (sh->endtime,my_wordorder);
    asc_sdr_time (str,sdr_time,my_wordorder);
    capntstr(hr.end_time, str, HR_END_TIME_LEN);
    unix_time = unix_time_from_sdr_time (sdr_time,my_wordorder);
    capntlong(hr.unix_end_time, unix_time, HR_UNIX_END_LEN);

    size_kb = ((out->num_blk * out->blksize) + 1024 - 1 ) / 1024;
/*::
    fprintf (stderr, "num_blk = %d, blksize = %d, kb = %d\n",
	     out->num_blk, out->blksize, size_kb);
::*/
    capntint(hr.file_kb,size_kb,HR_FILE_KB_LEN);

    capntstr(hr.dcc, dcc, HR_DCC_LEN);
    capntstr(hr.incoming_medium, "D", HR_MEDIUM_LEN);
    capntstr(hr.data_filename, filename, HR_DATA_FILE_LEN);
    capntstr(hr.header_dir, header_dir, HR_HEADER_DIR_LEN);
    capntstr(hr.operator, operator, HR_OPERATOR_LEN);

    /*	Convert final tab to a null since a holdings record consists	*/
    /*	of tab-separted fields and is terminated by a newline.		*/
    hr.operator[HR_OPERATOR_LEN] = '\0';
    puts((char *)&hr);
    fflush (stdout);
    return (0);
}

/************************************************************************/
/*  write_holdings_short:						*/
/*	Write short holding record (HOLDING_SHORT).			*/
/*  return:								*/
/*	0 on success, non-zero on failure.				*/
/************************************************************************/
int write_holdings_short
   (ST_INFO	*out)		/* ptr to output stream structure.	*/
{
    HOLDING_SHORT hr;		/* holdings record.			*/
    DATA_HDR	*sh = out->sum_hdr;
    char	str[80];
    SDR_TIME	sdr_time;
    char	*p;

    memset ((void *)&hr, 0, sizeof(hr));
    hr.len = sizeof (hr);
    hr.version = HOLDING_SHORT_VERSION;
    hr.blkexp = rint(log2(out->blksize));
    hr.data_type = out->data_type;
    strcpy (hr.station, sh->station_id);
    strcpy (hr.network, sh->network_id);
    strcpy (hr.channel, sh->channel_id);
    strcpy (hr.location, sh->location_id);
    hr.sample_rate = sh->sample_rate;
    hr.sample_rate_mult = sh->sample_rate_mult;
    hr.start_time = encode_time_sdr(sh->begtime,my_wordorder);
    hr.end_time = encode_time_sdr(sh->endtime,my_wordorder);
    hr.offset = out->init_offset;
    hr.length = out->cur_offset - out->init_offset;
    if (ascii_flag) {
	/* Substitute "-" for empty network and location field. */
	if (hr.network[0]=='\0') hr.network[0] = '-';
	if (hr.location[0]=='\0') hr.location[0] = '-';
	printf ("%d %d %d %d %-5.5s %-2.2s %-3.3s %-2.2s %4d %4d %s ", 
		hr.len, hr.version, hr.blkexp, hr.data_type,
		hr.station, hr.network, hr.channel, hr.location, 
		hr.sample_rate, hr.sample_rate_mult,
		time_to_str(decode_time_sdr(hr.start_time,my_wordorder), date_fmt));
	printf (" %s", time_to_str(decode_time_sdr(hr.end_time,my_wordorder), date_fmt));
	printf (" %d %d\n", hr.offset, hr.length);
/*::
	printf (" %d %d (%d %d)\n", hr.offset, hr.length, 
		hr.offset/out->blksize, hr.length/out->blksize);
::*/

    }
    else {
	fwrite (&hr, sizeof(hr), 1, stdout);
    }
    fflush (stdout);
    return (0);
}
